using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using Sharpility.Extensions;
using NodaTime;

namespace IO.Swagger.api.Models
{
    /// <summary>
    /// _
    /// </summary>
    public sealed class _:  IEquatable<_>
    { 
        /// <summary>
        /// Ограничение на количество почтовых ящиков. &#39;-&#39; означает отсутствие лимита.
        /// </summary>
        public string Mail { get; private set; }

        /// <summary>
        /// Ограничение на количество сайтов. &#39;-&#39; означает отсутствие лимита.
        /// </summary>
        public string Website { get; private set; }

        /// <summary>
        /// MySQL разрешен, если true.
        /// </summary>
        public bool? Mysql { get; private set; }

        /// <summary>
        /// Ограничение на размер дискового пространства. &#39;-&#39; означает отсутствие лимита.
        /// </summary>
        public string Size { get; private set; }

        /// <summary>
        /// MSSQL разрешен, если true.
        /// </summary>
        public bool? Mssql { get; private set; }

        /// <summary>
        /// CGI и PHP скрипты разрешены, если true.
        /// </summary>
        public bool? DynamicContent { get; private set; }

        /// <summary>
        /// Функция exec() в PHP разрешена, если true.
        /// </summary>
        public bool? PhpExec { get; private set; }

        /// <summary>
        /// Ограничение на количество дополнительных FTP логинов. &#39;-&#39; означает отсутствие лимита.
        /// </summary>
        public string Ftp2 { get; private set; }


        /// <summary>
        /// Empty constructor required by some serializers.
        /// Use _.Builder() for instance creation instead.
        /// </summary>
        [Obsolete]
        public _()
        {
        }

        private _(string Mail, string Website, bool? Mysql, string Size, bool? Mssql, bool? DynamicContent, bool? PhpExec, string Ftp2)
        {
            
            this.Mail = Mail;
            
            this.Website = Website;
            
            this.Mysql = Mysql;
            
            this.Size = Size;
            
            this.Mssql = Mssql;
            
            this.DynamicContent = DynamicContent;
            
            this.PhpExec = PhpExec;
            
            this.Ftp2 = Ftp2;
            
        }

        /// <summary>
        /// Returns builder of _.
        /// </summary>
        /// <returns>_Builder</returns>
        public static _Builder Builder()
        {
            return new _Builder();
        }

        /// <summary>
        /// Returns _Builder with properties set.
        /// Use it to change properties.
        /// </summary>
        /// <returns>_Builder</returns>
        public _Builder With()
        {
            return Builder()
                .Mail(Mail)
                .Website(Website)
                .Mysql(Mysql)
                .Size(Size)
                .Mssql(Mssql)
                .DynamicContent(DynamicContent)
                .PhpExec(PhpExec)
                .Ftp2(Ftp2);
        }

        public override string ToString()
        {
            return this.PropertiesToString();
        }

        public override bool Equals(object obj)
        {
            return this.EqualsByProperties(obj);
        }

        public bool Equals(_ other)
        {
            return Equals((object) other);
        }

        public override int GetHashCode()
        {
            return this.PropertiesHash();
        }

        /// <summary>
        /// Implementation of == operator for (_.
        /// </summary>
        /// <param name="left">Compared (_</param>
        /// <param name="right">Compared (_</param>
        /// <returns>true if compared items are equals, false otherwise</returns>
        public static bool operator == (_ left, _ right)
        {
            return Equals(left, right);
        }

        /// <summary>
        /// Implementation of != operator for (_.
        /// </summary>
        /// <param name="left">Compared (_</param>
        /// <param name="right">Compared (_</param>
        /// <returns>true if compared items are not equals, false otherwise</returns>
        public static bool operator != (_ left, _ right)
        {
            return !Equals(left, right);
        }

        /// <summary>
        /// Builder of _.
        /// </summary>
        public sealed class _Builder
        {
            private string _Mail;
            private string _Website;
            private bool? _Mysql;
            private string _Size;
            private bool? _Mssql;
            private bool? _DynamicContent;
            private bool? _PhpExec;
            private string _Ftp2;

            internal _Builder()
            {
                SetupDefaults();
            }

            private void SetupDefaults()
            {
            }

            /// <summary>
            /// Sets value for _.Mail property.
            /// </summary>
            /// <param name="value">Ограничение на количество почтовых ящиков. &#39;-&#39; означает отсутствие лимита.</param>
            public _Builder Mail(string value)
            {
                _Mail = value;
                return this;
            }

            /// <summary>
            /// Sets value for _.Website property.
            /// </summary>
            /// <param name="value">Ограничение на количество сайтов. &#39;-&#39; означает отсутствие лимита.</param>
            public _Builder Website(string value)
            {
                _Website = value;
                return this;
            }

            /// <summary>
            /// Sets value for _.Mysql property.
            /// </summary>
            /// <param name="value">MySQL разрешен, если true.</param>
            public _Builder Mysql(bool? value)
            {
                _Mysql = value;
                return this;
            }

            /// <summary>
            /// Sets value for _.Size property.
            /// </summary>
            /// <param name="value">Ограничение на размер дискового пространства. &#39;-&#39; означает отсутствие лимита.</param>
            public _Builder Size(string value)
            {
                _Size = value;
                return this;
            }

            /// <summary>
            /// Sets value for _.Mssql property.
            /// </summary>
            /// <param name="value">MSSQL разрешен, если true.</param>
            public _Builder Mssql(bool? value)
            {
                _Mssql = value;
                return this;
            }

            /// <summary>
            /// Sets value for _.DynamicContent property.
            /// </summary>
            /// <param name="value">CGI и PHP скрипты разрешены, если true.</param>
            public _Builder DynamicContent(bool? value)
            {
                _DynamicContent = value;
                return this;
            }

            /// <summary>
            /// Sets value for _.PhpExec property.
            /// </summary>
            /// <param name="value">Функция exec() в PHP разрешена, если true.</param>
            public _Builder PhpExec(bool? value)
            {
                _PhpExec = value;
                return this;
            }

            /// <summary>
            /// Sets value for _.Ftp2 property.
            /// </summary>
            /// <param name="value">Ограничение на количество дополнительных FTP логинов. &#39;-&#39; означает отсутствие лимита.</param>
            public _Builder Ftp2(string value)
            {
                _Ftp2 = value;
                return this;
            }


            /// <summary>
            /// Builds instance of _.
            /// </summary>
            /// <returns>_</returns>
            public _ Build()
            {
                Validate();
                return new _(
                    Mail: _Mail,
                    Website: _Website,
                    Mysql: _Mysql,
                    Size: _Size,
                    Mssql: _Mssql,
                    DynamicContent: _DynamicContent,
                    PhpExec: _PhpExec,
                    Ftp2: _Ftp2
                );
            }

            private void Validate()
            { 
                if (_Mail == null)
                {
                    throw new ArgumentException("Mail is a required property for _ and cannot be null");
                } 
                if (_Website == null)
                {
                    throw new ArgumentException("Website is a required property for _ and cannot be null");
                } 
                if (_Mysql == null)
                {
                    throw new ArgumentException("Mysql is a required property for _ and cannot be null");
                } 
                if (_Size == null)
                {
                    throw new ArgumentException("Size is a required property for _ and cannot be null");
                } 
                if (_Mssql == null)
                {
                    throw new ArgumentException("Mssql is a required property for _ and cannot be null");
                } 
                if (_DynamicContent == null)
                {
                    throw new ArgumentException("DynamicContent is a required property for _ and cannot be null");
                } 
                if (_PhpExec == null)
                {
                    throw new ArgumentException("PhpExec is a required property for _ and cannot be null");
                } 
                if (_Ftp2 == null)
                {
                    throw new ArgumentException("Ftp2 is a required property for _ and cannot be null");
                } 
            }
        }

        
    }
}
