using System;
using Nancy;
using Nancy.ModelBinding;
using System.Collections.Generic;
using Sharpility.Base;
using IO.Swagger.api.Models;
using IO.Swagger.api.Utils;
using NodaTime;

namespace IO.Swagger.api.Modules
{ 

    /// <summary>
    /// Module processing requests of DNS domain.
    /// </summary>
    public sealed class DNSModule : NancyModule
    {
        /// <summary>
        /// Sets up HTTP methods mappings.
        /// </summary>
        /// <param name="service">Service handling requests</param>
        public DNSModule(DNSService service) : base("/api")
        { 
            Get["/dns/add"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                var addDnsZone = Parameters.ValueOf<string>(parameters, Context.Request, "addDnsZone", ParameterType.Query);
                var addDnsRecord = Parameters.ValueOf<string>(parameters, Context.Request, "addDnsRecord", ParameterType.Query);
                var iisSiteForDns = Parameters.ValueOf<string>(parameters, Context.Request, "iisSiteForDns", ParameterType.Query);
                var bindOnly = Parameters.ValueOf<bool?>(parameters, Context.Request, "bindOnly", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'DnsAddGet'");
                
                Preconditions.IsNotNull(addDnsZone, "Required parameter: 'addDnsZone' is missing at 'DnsAddGet'");
                
                return service.DnsAddGet(Context, token_, addDnsZone, addDnsRecord, iisSiteForDns, bindOnly);
            };

            Get["/dns/bind"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                var key_ = Parameters.ValueOf<int?>(parameters, Context.Request, "key_", ParameterType.Query);
                var bindTo = Parameters.ValueOf<string>(parameters, Context.Request, "bindTo", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'DnsBindGet'");
                
                Preconditions.IsNotNull(key_, "Required parameter: 'key_' is missing at 'DnsBindGet'");
                
                return service.DnsBindGet(Context, token_, key_, bindTo);
            };

            Get["/dns/list"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'DnsListGet'");
                
                return service.DnsListGet(Context, token_);
            };

            Get["/dns/prolong/list"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'DnsProlongListGet'");
                
                return service.DnsProlongListGet(Context, token_);
            };

            Get["/dns/raw"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                var key_ = Parameters.ValueOf<int?>(parameters, Context.Request, "key_", ParameterType.Query);
                var sDel = Parameters.ValueOf<int?>(parameters, Context.Request, "sDel", ParameterType.Query);
                var sAdd = Parameters.ValueOf<int?>(parameters, Context.Request, "sAdd", ParameterType.Query);
                var dnsType = Parameters.ValueOf<string>(parameters, Context.Request, "dnsType", ParameterType.Query);
                var dnsName = Parameters.ValueOf<string>(parameters, Context.Request, "dnsName", ParameterType.Query);
                var dnsValue = Parameters.ValueOf<string>(parameters, Context.Request, "dnsValue", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'DnsRawGet'");
                
                Preconditions.IsNotNull(key_, "Required parameter: 'key_' is missing at 'DnsRawGet'");
                
                Preconditions.IsNotNull(sDel, "Required parameter: 'sDel' is missing at 'DnsRawGet'");
                
                Preconditions.IsNotNull(sAdd, "Required parameter: 'sAdd' is missing at 'DnsRawGet'");
                
                Preconditions.IsNotNull(dnsType, "Required parameter: 'dnsType' is missing at 'DnsRawGet'");
                
                Preconditions.IsNotNull(dnsName, "Required parameter: 'dnsName' is missing at 'DnsRawGet'");
                
                Preconditions.IsNotNull(dnsValue, "Required parameter: 'dnsValue' is missing at 'DnsRawGet'");
                
                return service.DnsRawGet(Context, token_, key_, sDel, sAdd, dnsType, dnsName, dnsValue);
            };

            Get["/dns/raw/managedmx"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                var key_ = Parameters.ValueOf<int?>(parameters, Context.Request, "key_", ParameterType.Query);
                var managedMx = Parameters.ValueOf<int?>(parameters, Context.Request, "managedMx", ParameterType.Query);
                var updManagedMx = Parameters.ValueOf<int?>(parameters, Context.Request, "updManagedMx", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'DnsRawManagedmxGet'");
                
                Preconditions.IsNotNull(key_, "Required parameter: 'key_' is missing at 'DnsRawManagedmxGet'");
                
                Preconditions.IsNotNull(managedMx, "Required parameter: 'managedMx' is missing at 'DnsRawManagedmxGet'");
                
                Preconditions.IsNotNull(updManagedMx, "Required parameter: 'updManagedMx' is missing at 'DnsRawManagedmxGet'");
                
                return service.DnsRawManagedmxGet(Context, token_, key_, managedMx, updManagedMx);
            };
        }
    }

    /// <summary>
    /// Service handling DNS requests.
    /// </summary>
    public interface DNSService
    {
        /// <summary>
        /// 
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <param name="addDnsZone">зона, которую необходимо создать</param>
        /// <param name="addDnsRecord"> (optional)</param>
        /// <param name="iisSiteForDns">ID сайта из /website/list, либо IP-адрес виртуального сервера, к которому необходимо сделать привязку (optional)</param>
        /// <param name="bindOnly"> (optional)</param>
        /// <returns>string</returns>
        string DnsAddGet(NancyContext context, string token_, string addDnsZone, string addDnsRecord, string iisSiteForDns, bool? bindOnly);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <param name="key_">ID домена из списка /dns/list</param>
        /// <param name="bindTo">домен, к которому необходимо сделать привязку (optional)</param>
        /// <returns>string</returns>
        string DnsBindGet(NancyContext context, string token_, int? key_, string bindTo);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <returns>List&lt;InlineResponse2003&gt;</returns>
        List<InlineResponse2003> DnsListGet(NancyContext context, string token_);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <returns>List&lt;InlineResponse2004&gt;</returns>
        List<InlineResponse2004> DnsProlongListGet(NancyContext context, string token_);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <param name="key_">ID домена из /dns/list</param>
        /// <param name="sDel">Значение &#39;1&#39; сообщает, что запись необходимо удалить</param>
        /// <param name="sAdd">Значение &#39;1&#39; сообщает, что запись необходимо создать</param>
        /// <param name="dnsType">Тип записи: A, MX, CNAME, AAAA, NS, TXT, SRV</param>
        /// <param name="dnsName">Имя записи, либо &#39;@&#39; для корневой</param>
        /// <param name="dnsValue">Содержание записи (для A это IP-адрес, для CNAME имя, для MX приоритет и назначение)</param>
        /// <returns>string</returns>
        string DnsRawGet(NancyContext context, string token_, int? key_, int? sDel, int? sAdd, string dnsType, string dnsName, string dnsValue);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <param name="key_">ID домена из /dns/list</param>
        /// <param name="managedMx">Значение &#39;1&#39;, если необходимо включить поддержку на MX-серверах хостинга и &#39;0&#39;, если выключить</param>
        /// <param name="updManagedMx">Значение &#39;1&#39; подтверждает операцию</param>
        /// <returns>string</returns>
        string DnsRawManagedmxGet(NancyContext context, string token_, int? key_, int? managedMx, int? updManagedMx);
    }

    /// <summary>
    /// Abstraction of DNSService.
    /// </summary>
    public abstract class AbstractDNSService: DNSService
    {
        public virtual string DnsAddGet(NancyContext context, string token_, string addDnsZone, string addDnsRecord, string iisSiteForDns, bool? bindOnly)
        {
            return DnsAddGet(token_, addDnsZone, addDnsRecord, iisSiteForDns, bindOnly);
        }

        public virtual string DnsBindGet(NancyContext context, string token_, int? key_, string bindTo)
        {
            return DnsBindGet(token_, key_, bindTo);
        }

        public virtual List<InlineResponse2003> DnsListGet(NancyContext context, string token_)
        {
            return DnsListGet(token_);
        }

        public virtual List<InlineResponse2004> DnsProlongListGet(NancyContext context, string token_)
        {
            return DnsProlongListGet(token_);
        }

        public virtual string DnsRawGet(NancyContext context, string token_, int? key_, int? sDel, int? sAdd, string dnsType, string dnsName, string dnsValue)
        {
            return DnsRawGet(token_, key_, sDel, sAdd, dnsType, dnsName, dnsValue);
        }

        public virtual string DnsRawManagedmxGet(NancyContext context, string token_, int? key_, int? managedMx, int? updManagedMx)
        {
            return DnsRawManagedmxGet(token_, key_, managedMx, updManagedMx);
        }

        protected abstract string DnsAddGet(string token_, string addDnsZone, string addDnsRecord, string iisSiteForDns, bool? bindOnly);

        protected abstract string DnsBindGet(string token_, int? key_, string bindTo);

        protected abstract List<InlineResponse2003> DnsListGet(string token_);

        protected abstract List<InlineResponse2004> DnsProlongListGet(string token_);

        protected abstract string DnsRawGet(string token_, int? key_, int? sDel, int? sAdd, string dnsType, string dnsName, string dnsValue);

        protected abstract string DnsRawManagedmxGet(string token_, int? key_, int? managedMx, int? updManagedMx);
    }

}
