using System;
using Nancy;
using Nancy.ModelBinding;
using System.Collections.Generic;
using Sharpility.Base;
using IO.Swagger.api.Models;
using IO.Swagger.api.Utils;
using NodaTime;

namespace IO.Swagger.api.Modules
{ 

    /// <summary>
    /// Module processing requests of Default domain.
    /// </summary>
    public sealed class DefaultModule : NancyModule
    {
        /// <summary>
        /// Sets up HTTP methods mappings.
        /// </summary>
        /// <param name="service">Service handling requests</param>
        public DefaultModule(DefaultService service) : base("/api")
        { 
            Get["/balance"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'BalanceGet'");
                
                return service.BalanceGet(Context, token_);
            };

            Get["/balance.lic"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'BalanceLicGet'");
                
                return service.BalanceLicGet(Context, token_);
            };

            Get["/billing/debt"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'BillingDebtGet'");
                
                return service.BillingDebtGet(Context, token_);
            };

            Get["/billing/info"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'BillingInfoGet'");
                
                return service.BillingInfoGet(Context, token_);
            };

            Get["/billing/limits"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'BillingLimitsGet'");
                
                return service.BillingLimitsGet(Context, token_);
            };

            Get["/dns/prolong/list"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'DnsProlongListGet'");
                
                return service.DnsProlongListGet(Context, token_);
            };

            Get["/email/add"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                var addEmail = Parameters.ValueOf<string>(parameters, Context.Request, "addEmail", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'EmailAddGet'");
                
                Preconditions.IsNotNull(addEmail, "Required parameter: 'addEmail' is missing at 'EmailAddGet'");
                
                return service.EmailAddGet(Context, token_, addEmail);
            };

            Get["/email/alias/delete"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                var key_ = Parameters.ValueOf<string>(parameters, Context.Request, "key_", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'EmailAliasDeleteGet'");
                
                Preconditions.IsNotNull(key_, "Required parameter: 'key_' is missing at 'EmailAliasDeleteGet'");
                
                return service.EmailAliasDeleteGet(Context, token_, key_);
            };

            Get["/email/alias/list"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'EmailAliasListGet'");
                
                return service.EmailAliasListGet(Context, token_);
            };

            Get["/email/delete"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                var key_ = Parameters.ValueOf<string>(parameters, Context.Request, "key_", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'EmailDeleteGet'");
                
                Preconditions.IsNotNull(key_, "Required parameter: 'key_' is missing at 'EmailDeleteGet'");
                
                return service.EmailDeleteGet(Context, token_, key_);
            };

            Get["/email/list"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'EmailListGet'");
                
                return service.EmailListGet(Context, token_);
            };

            Get["/updates/last"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'UpdatesLastGet'");
                
                return service.UpdatesLastGet(Context, token_);
            };

            Get["/updates/list"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                var key_ = Parameters.ValueOf<int?>(parameters, Context.Request, "key_", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'UpdatesListGet'");
                
                Preconditions.IsNotNull(key_, "Required parameter: 'key_' is missing at 'UpdatesListGet'");
                
                return service.UpdatesListGet(Context, token_, key_);
            };

            Get["/website/delete"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                var key_ = Parameters.ValueOf<string>(parameters, Context.Request, "key_", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'WebsiteDeleteGet'");
                
                Preconditions.IsNotNull(key_, "Required parameter: 'key_' is missing at 'WebsiteDeleteGet'");
                
                return service.WebsiteDeleteGet(Context, token_, key_);
            };

            Get["/website/list"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'WebsiteListGet'");
                
                return service.WebsiteListGet(Context, token_);
            };
        }
    }

    /// <summary>
    /// Service handling Default requests.
    /// </summary>
    public interface DefaultService
    {
        /// <summary>
        /// Возвращает текущий баланс аккаунта
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <returns>decimal?</returns>
        decimal? BalanceGet(NancyContext context, string token_);

        /// <summary>
        /// Возвращает текущий баланс аккаунта лицензий
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <returns>decimal?</returns>
        decimal? BalanceLicGet(NancyContext context, string token_);

        /// <summary>
        /// Данный запрос позволяет получить информацию по планируемым датам наступления негативного баланса и отключения
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <returns>List&lt;InlineResponse200&gt;</returns>
        List<InlineResponse200> BillingDebtGet(NancyContext context, string token_);

        /// <summary>
        /// Этот запрос позволяет получить информацию об аккаунте, а также базовую статистику использования ресурсов.
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <returns>List&lt;InlineResponse2001&gt;</returns>
        List<InlineResponse2001> BillingInfoGet(NancyContext context, string token_);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <returns>_</returns>
        _ BillingLimitsGet(NancyContext context, string token_);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <returns>List&lt;InlineResponse2004&gt;</returns>
        List<InlineResponse2004> DnsProlongListGet(NancyContext context, string token_);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <param name="addEmail">Адрес почтового ящика, который необходимо создать</param>
        /// <returns>string</returns>
        string EmailAddGet(NancyContext context, string token_, string addEmail);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <param name="key_">Адрес почтового ящика, подлежащего удалению</param>
        /// <returns>string</returns>
        string EmailAliasDeleteGet(NancyContext context, string token_, string key_);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <returns>List&lt;InlineResponse2005&gt;</returns>
        List<InlineResponse2005> EmailAliasListGet(NancyContext context, string token_);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <param name="key_">Адрес почтового ящика, подлежащего удалению</param>
        /// <returns>string</returns>
        string EmailDeleteGet(NancyContext context, string token_, string key_);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <returns>List&lt;InlineResponse2006&gt;</returns>
        List<InlineResponse2006> EmailListGet(NancyContext context, string token_);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <returns>int?</returns>
        int? UpdatesLastGet(NancyContext context, string token_);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <param name="key_">ID первой записи для показа или 0 для показа всех записей</param>
        /// <returns>List&lt;InlineResponse20010&gt;</returns>
        List<InlineResponse20010> UpdatesListGet(NancyContext context, string token_, int? key_);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <param name="key_">ID сайта, полученный из /website/list</param>
        /// <returns>string</returns>
        string WebsiteDeleteGet(NancyContext context, string token_, string key_);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <returns>List&lt;InlineResponse20014&gt;</returns>
        List<InlineResponse20014> WebsiteListGet(NancyContext context, string token_);
    }

    /// <summary>
    /// Abstraction of DefaultService.
    /// </summary>
    public abstract class AbstractDefaultService: DefaultService
    {
        public virtual decimal? BalanceGet(NancyContext context, string token_)
        {
            return BalanceGet(token_);
        }

        public virtual decimal? BalanceLicGet(NancyContext context, string token_)
        {
            return BalanceLicGet(token_);
        }

        public virtual List<InlineResponse200> BillingDebtGet(NancyContext context, string token_)
        {
            return BillingDebtGet(token_);
        }

        public virtual List<InlineResponse2001> BillingInfoGet(NancyContext context, string token_)
        {
            return BillingInfoGet(token_);
        }

        public virtual _ BillingLimitsGet(NancyContext context, string token_)
        {
            return BillingLimitsGet(token_);
        }

        public virtual List<InlineResponse2004> DnsProlongListGet(NancyContext context, string token_)
        {
            return DnsProlongListGet(token_);
        }

        public virtual string EmailAddGet(NancyContext context, string token_, string addEmail)
        {
            return EmailAddGet(token_, addEmail);
        }

        public virtual string EmailAliasDeleteGet(NancyContext context, string token_, string key_)
        {
            return EmailAliasDeleteGet(token_, key_);
        }

        public virtual List<InlineResponse2005> EmailAliasListGet(NancyContext context, string token_)
        {
            return EmailAliasListGet(token_);
        }

        public virtual string EmailDeleteGet(NancyContext context, string token_, string key_)
        {
            return EmailDeleteGet(token_, key_);
        }

        public virtual List<InlineResponse2006> EmailListGet(NancyContext context, string token_)
        {
            return EmailListGet(token_);
        }

        public virtual int? UpdatesLastGet(NancyContext context, string token_)
        {
            return UpdatesLastGet(token_);
        }

        public virtual List<InlineResponse20010> UpdatesListGet(NancyContext context, string token_, int? key_)
        {
            return UpdatesListGet(token_, key_);
        }

        public virtual string WebsiteDeleteGet(NancyContext context, string token_, string key_)
        {
            return WebsiteDeleteGet(token_, key_);
        }

        public virtual List<InlineResponse20014> WebsiteListGet(NancyContext context, string token_)
        {
            return WebsiteListGet(token_);
        }

        protected abstract decimal? BalanceGet(string token_);

        protected abstract decimal? BalanceLicGet(string token_);

        protected abstract List<InlineResponse200> BillingDebtGet(string token_);

        protected abstract List<InlineResponse2001> BillingInfoGet(string token_);

        protected abstract _ BillingLimitsGet(string token_);

        protected abstract List<InlineResponse2004> DnsProlongListGet(string token_);

        protected abstract string EmailAddGet(string token_, string addEmail);

        protected abstract string EmailAliasDeleteGet(string token_, string key_);

        protected abstract List<InlineResponse2005> EmailAliasListGet(string token_);

        protected abstract string EmailDeleteGet(string token_, string key_);

        protected abstract List<InlineResponse2006> EmailListGet(string token_);

        protected abstract int? UpdatesLastGet(string token_);

        protected abstract List<InlineResponse20010> UpdatesListGet(string token_, int? key_);

        protected abstract string WebsiteDeleteGet(string token_, string key_);

        protected abstract List<InlineResponse20014> WebsiteListGet(string token_);
    }

}
