using System;
using Nancy;
using Nancy.ModelBinding;
using System.Collections.Generic;
using Sharpility.Base;
using IO.Swagger.api.Models;
using IO.Swagger.api.Utils;
using NodaTime;

namespace IO.Swagger.api.Modules
{ 

    /// <summary>
    /// Module processing requests of HostingftpAPI domain.
    /// </summary>
    public sealed class HostingftpAPIModule : NancyModule
    {
        /// <summary>
        /// Sets up HTTP methods mappings.
        /// </summary>
        /// <param name="service">Service handling requests</param>
        public HostingftpAPIModule(HostingftpAPIService service) : base("/api")
        { 
            Get["/ftp/list"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'FtpListGet'");
                
                return service.FtpListGet(Context, token_);
            };
        }
    }

    /// <summary>
    /// Service handling HostingftpAPI requests.
    /// </summary>
    public interface HostingftpAPIService
    {
        /// <summary>
        /// 
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <returns>List&lt;InlineResponse2007&gt;</returns>
        List<InlineResponse2007> FtpListGet(NancyContext context, string token_);
    }

    /// <summary>
    /// Abstraction of HostingftpAPIService.
    /// </summary>
    public abstract class AbstractHostingftpAPIService: HostingftpAPIService
    {
        public virtual List<InlineResponse2007> FtpListGet(NancyContext context, string token_)
        {
            return FtpListGet(token_);
        }

        protected abstract List<InlineResponse2007> FtpListGet(string token_);
    }

}
