using System;
using Nancy;
using Nancy.ModelBinding;
using System.Collections.Generic;
using Sharpility.Base;
using IO.Swagger.api.Models;
using IO.Swagger.api.Utils;
using NodaTime;

namespace IO.Swagger.api.Modules
{ 

    /// <summary>
    /// Module processing requests of HostingiisAPI domain.
    /// </summary>
    public sealed class HostingiisAPIModule : NancyModule
    {
        /// <summary>
        /// Sets up HTTP methods mappings.
        /// </summary>
        /// <param name="service">Service handling requests</param>
        public HostingiisAPIModule(HostingiisAPIService service) : base("/api")
        { 
            Get["/iis/list"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'IisListGet'");
                
                return service.IisListGet(Context, token_);
            };
        }
    }

    /// <summary>
    /// Service handling HostingiisAPI requests.
    /// </summary>
    public interface HostingiisAPIService
    {
        /// <summary>
        /// 
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <returns>List&lt;InlineResponse2008&gt;</returns>
        List<InlineResponse2008> IisListGet(NancyContext context, string token_);
    }

    /// <summary>
    /// Abstraction of HostingiisAPIService.
    /// </summary>
    public abstract class AbstractHostingiisAPIService: HostingiisAPIService
    {
        public virtual List<InlineResponse2008> IisListGet(NancyContext context, string token_)
        {
            return IisListGet(token_);
        }

        protected abstract List<InlineResponse2008> IisListGet(string token_);
    }

}
