using System;
using Nancy;
using Nancy.ModelBinding;
using System.Collections.Generic;
using Sharpility.Base;
using IO.Swagger.api.Models;
using IO.Swagger.api.Utils;
using NodaTime;

namespace IO.Swagger.api.Modules
{ 

    /// <summary>
    /// Module processing requests of MicrosoftSQLServer domain.
    /// </summary>
    public sealed class MicrosoftSQLServerModule : NancyModule
    {
        /// <summary>
        /// Sets up HTTP methods mappings.
        /// </summary>
        /// <param name="service">Service handling requests</param>
        public MicrosoftSQLServerModule(MicrosoftSQLServerService service) : base("/api")
        { 
            Get["/mssql/list"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'MssqlListGet'");
                
                return service.MssqlListGet(Context, token_);
            };
        }
    }

    /// <summary>
    /// Service handling MicrosoftSQLServer requests.
    /// </summary>
    public interface MicrosoftSQLServerService
    {
        /// <summary>
        /// 
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <returns>List&lt;InlineResponse2009&gt;</returns>
        List<InlineResponse2009> MssqlListGet(NancyContext context, string token_);
    }

    /// <summary>
    /// Abstraction of MicrosoftSQLServerService.
    /// </summary>
    public abstract class AbstractMicrosoftSQLServerService: MicrosoftSQLServerService
    {
        public virtual List<InlineResponse2009> MssqlListGet(NancyContext context, string token_)
        {
            return MssqlListGet(token_);
        }

        protected abstract List<InlineResponse2009> MssqlListGet(string token_);
    }

}
