using System;
using Nancy;
using Nancy.ModelBinding;
using System.Collections.Generic;
using Sharpility.Base;
using IO.Swagger.api.Models;
using IO.Swagger.api.Utils;
using NodaTime;

namespace IO.Swagger.api.Modules
{ 

    /// <summary>
    /// Module processing requests of MySQLPostgres domain.
    /// </summary>
    public sealed class MySQLPostgresModule : NancyModule
    {
        /// <summary>
        /// Sets up HTTP methods mappings.
        /// </summary>
        /// <param name="service">Service handling requests</param>
        public MySQLPostgresModule(MySQLPostgresService service) : base("/api")
        { 
            Get["/database/list"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'DatabaseListGet'");
                
                return service.DatabaseListGet(Context, token_);
            };
        }
    }

    /// <summary>
    /// Service handling MySQLPostgres requests.
    /// </summary>
    public interface MySQLPostgresService
    {
        /// <summary>
        /// Возвращает список зарегистрированных аккаунтов в базах данных MySQL и Postgres. Для вычисления имени домена необходимо объединить имя базы данных,   идентификатор сервера и суффикс (поле url_suffix). Например, если ID сервера равен 123 (значит это MySQL — см. описание поля &#39;server&#39; в ответе), а суффикс равен &#39;1gb.ru&#39;, то домен   для доступа будет равен &#39;mysql123.1gb.ru&#39;.
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <returns>List&lt;InlineResponse2002&gt;</returns>
        List<InlineResponse2002> DatabaseListGet(NancyContext context, string token_);
    }

    /// <summary>
    /// Abstraction of MySQLPostgresService.
    /// </summary>
    public abstract class AbstractMySQLPostgresService: MySQLPostgresService
    {
        public virtual List<InlineResponse2002> DatabaseListGet(NancyContext context, string token_)
        {
            return DatabaseListGet(token_);
        }

        protected abstract List<InlineResponse2002> DatabaseListGet(string token_);
    }

}
