using System;
using Nancy;
using Nancy.ModelBinding;
using System.Collections.Generic;
using Sharpility.Base;
using IO.Swagger.api.Models;
using IO.Swagger.api.Utils;
using NodaTime;

namespace IO.Swagger.api.Modules
{ 

    /// <summary>
    /// Module processing requests of VDSHyperVOpenVZ domain.
    /// </summary>
    public sealed class VDSHyperVOpenVZModule : NancyModule
    {
        /// <summary>
        /// Sets up HTTP methods mappings.
        /// </summary>
        /// <param name="service">Service handling requests</param>
        public VDSHyperVOpenVZModule(VDSHyperVOpenVZService service) : base("/api")
        { 
            Get["/vds/create"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                var type = Parameters.ValueOf<string>(parameters, Context.Request, "type", ParameterType.Query);
                var crHvmem = Parameters.ValueOf<int?>(parameters, Context.Request, "crHvmem", ParameterType.Query);
                var crHvdsk = Parameters.ValueOf<int?>(parameters, Context.Request, "crHvdsk", ParameterType.Query);
                var crHvcpu = Parameters.ValueOf<int?>(parameters, Context.Request, "crHvcpu", ParameterType.Query);
                var template = Parameters.ValueOf<string>(parameters, Context.Request, "template", ParameterType.Query);
                var forceMemo = Parameters.ValueOf<string>(parameters, Context.Request, "forceMemo", ParameterType.Query);
                var hv1gbPanelVersion = Parameters.ValueOf<string>(parameters, Context.Request, "hv1gbPanelVersion", ParameterType.Query);
                var panelInstall = Parameters.ValueOf<bool?>(parameters, Context.Request, "panelInstall", ParameterType.Query);
                var crSsd1 = Parameters.ValueOf<bool?>(parameters, Context.Request, "crSsd1", ParameterType.Query);
                var crHvAllSystem = Parameters.ValueOf<bool?>(parameters, Context.Request, "crHvAllSystem", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'VdsCreateGet'");
                
                return service.VdsCreateGet(Context, token_, type, crHvmem, crHvdsk, crHvcpu, template, forceMemo, hv1gbPanelVersion, panelInstall, crSsd1, crHvAllSystem);
            };

            Get["/vds/delete"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                var key_ = Parameters.ValueOf<int?>(parameters, Context.Request, "key_", ParameterType.Query);
                var forceCritical = Parameters.ValueOf<bool?>(parameters, Context.Request, "forceCritical", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'VdsDeleteGet'");
                
                Preconditions.IsNotNull(key_, "Required parameter: 'key_' is missing at 'VdsDeleteGet'");
                
                return service.VdsDeleteGet(Context, token_, key_, forceCritical);
            };

            Get["/vds/ip/add"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                var key_ = Parameters.ValueOf<int?>(parameters, Context.Request, "key_", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'VdsIpAddGet'");
                
                Preconditions.IsNotNull(key_, "Required parameter: 'key_' is missing at 'VdsIpAddGet'");
                
                return service.VdsIpAddGet(Context, token_, key_);
            };

            Get["/vds/ip/delete"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                var key_ = Parameters.ValueOf<int?>(parameters, Context.Request, "key_", ParameterType.Query);
                var delIp = Parameters.ValueOf<string>(parameters, Context.Request, "delIp", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'VdsIpDeleteGet'");
                
                Preconditions.IsNotNull(key_, "Required parameter: 'key_' is missing at 'VdsIpDeleteGet'");
                
                Preconditions.IsNotNull(delIp, "Required parameter: 'delIp' is missing at 'VdsIpDeleteGet'");
                
                return service.VdsIpDeleteGet(Context, token_, key_, delIp);
            };

            Get["/vds/ip/list"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                var key_ = Parameters.ValueOf<int?>(parameters, Context.Request, "key_", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'VdsIpListGet'");
                
                return service.VdsIpListGet(Context, token_, key_);
            };

            Get["/vds/list"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                var key_ = Parameters.ValueOf<int?>(parameters, Context.Request, "key_", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'VdsListGet'");
                
                return service.VdsListGet(Context, token_, key_);
            };

            Get["/vds/reboot"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                var key_ = Parameters.ValueOf<int?>(parameters, Context.Request, "key_", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'VdsRebootGet'");
                
                Preconditions.IsNotNull(key_, "Required parameter: 'key_' is missing at 'VdsRebootGet'");
                
                return service.VdsRebootGet(Context, token_, key_);
            };

            Get["/vds/templates-list"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'VdsTemplatesListGet'");
                
                return service.VdsTemplatesListGet(Context, token_);
            };

            Get["/vds/update"] = parameters =>
            {
                var token_ = Parameters.ValueOf<string>(parameters, Context.Request, "token_", ParameterType.Query);
                var key_ = Parameters.ValueOf<int?>(parameters, Context.Request, "key_", ParameterType.Query);
                var crHvmem = Parameters.ValueOf<int?>(parameters, Context.Request, "crHvmem", ParameterType.Query);
                var crHvdsk = Parameters.ValueOf<int?>(parameters, Context.Request, "crHvdsk", ParameterType.Query);
                var crHvcpu = Parameters.ValueOf<int?>(parameters, Context.Request, "crHvcpu", ParameterType.Query);
                Preconditions.IsNotNull(token_, "Required parameter: 'token_' is missing at 'VdsUpdateGet'");
                
                Preconditions.IsNotNull(key_, "Required parameter: 'key_' is missing at 'VdsUpdateGet'");
                
                return service.VdsUpdateGet(Context, token_, key_, crHvmem, crHvdsk, crHvcpu);
            };
        }
    }

    /// <summary>
    /// Service handling VDSHyperVOpenVZ requests.
    /// </summary>
    public interface VDSHyperVOpenVZService
    {
        /// <summary>
        /// Изменение параметров уже существующего VDS
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <param name="type">Тип создаваемого VDS. &#39;vz1&#39;, &#39;vz2&#39;, &#39;vz3&#39;, &#39;vz4&#39;, &#39;vz5&#39; или &#39;vds.dynamic&#39;. По умолчанию &#39;vds.dynamic&#39; (optional)</param>
        /// <param name="crHvmem">Количество доступной памяти в мегабайтах. По умолчанию 2048 (optional)</param>
        /// <param name="crHvdsk">Количество доступных гигабайт диска. По умолчанию 30 (optional)</param>
        /// <param name="crHvcpu">Количество ядер CPU. По умолчанию 2 (optional)</param>
        /// <param name="template">Шаблон и тип виртуализации VDS. За подробностями см. /vds/templates-list (optional)</param>
        /// <param name="forceMemo"> (optional)</param>
        /// <param name="hv1gbPanelVersion"> (optional)</param>
        /// <param name="panelInstall">Необходимо ли установить панель 1Gb на VDS (optional)</param>
        /// <param name="crSsd1">Создать всю систему на SSD (optional)</param>
        /// <param name="crHvAllSystem">Создать всю систему на одном диске большого размера (иначе — системный диск + диск с данными) (optional)</param>
        /// <returns>string</returns>
        string VdsCreateGet(NancyContext context, string token_, string type, int? crHvmem, int? crHvdsk, int? crHvcpu, string template, string forceMemo, string hv1gbPanelVersion, bool? panelInstall, bool? crSsd1, bool? crHvAllSystem);

        /// <summary>
        /// Предоставляет функционал для удаления VDS
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <param name="key_">ID виртуального сервера, полученный из /vds/list</param>
        /// <param name="forceCritical">Если true, VDS будет удален срочно (optional)</param>
        /// <returns>string</returns>
        string VdsDeleteGet(NancyContext context, string token_, int? key_, bool? forceCritical);

        /// <summary>
        /// Этот вызов позволяет добавить ещё один IP адрес на указанный VDS
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <param name="key_">ID виртуального сервера, полученный из /vds/list</param>
        /// <returns>string</returns>
        string VdsIpAddGet(NancyContext context, string token_, int? key_);

        /// <summary>
        /// Этот вызов позволяет отвязать IP адрес от VDS
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <param name="key_">ID виртуального сервера, полученный из /vds/list</param>
        /// <param name="delIp">IP адрес, который необходимо удалить</param>
        /// <returns>string</returns>
        string VdsIpDeleteGet(NancyContext context, string token_, int? key_, string delIp);

        /// <summary>
        /// Список IP-адресов виртуальных серверов
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <param name="key_">ID виртуального сервера, полученный из /vds/list (optional)</param>
        /// <returns>List&lt;InlineResponse20011&gt;</returns>
        List<InlineResponse20011> VdsIpListGet(NancyContext context, string token_, int? key_);

        /// <summary>
        /// Позволяет получить список виртуальных серверов
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <param name="key_">ID виртуального сервера. Если задан, то будет выдана информация только для этого VDS (optional)</param>
        /// <returns>List&lt;InlineResponse20012&gt;</returns>
        List<InlineResponse20012> VdsListGet(NancyContext context, string token_, int? key_);

        /// <summary>
        /// Предоставляет функционал для перезагрузки VDS
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <param name="key_">ID виртуального сервера, полученный из /vds/list</param>
        /// <returns>string</returns>
        string VdsRebootGet(NancyContext context, string token_, int? key_);

        /// <summary>
        /// Позволяет получить список шаблонов виртуальных серверов
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <returns>List&lt;InlineResponse20013&gt;</returns>
        List<InlineResponse20013> VdsTemplatesListGet(NancyContext context, string token_);

        /// <summary>
        /// Создание новых VDS с произвольными характеристиками
        /// </summary>
        /// <param name="context">Context of request</param>
        /// <param name="token_">Ключ доступа, полученный при авторизации</param>
        /// <param name="key_">ID виртуального сервера, полученный из /vds/list</param>
        /// <param name="crHvmem">Количество доступной памяти в мегабайтах. По умолчанию 2048 (optional)</param>
        /// <param name="crHvdsk">Количество доступных гигабайт диска. По умолчанию 30 (optional)</param>
        /// <param name="crHvcpu">Количество ядер CPU. По умолчанию 2 (optional)</param>
        /// <returns>string</returns>
        string VdsUpdateGet(NancyContext context, string token_, int? key_, int? crHvmem, int? crHvdsk, int? crHvcpu);
    }

    /// <summary>
    /// Abstraction of VDSHyperVOpenVZService.
    /// </summary>
    public abstract class AbstractVDSHyperVOpenVZService: VDSHyperVOpenVZService
    {
        public virtual string VdsCreateGet(NancyContext context, string token_, string type, int? crHvmem, int? crHvdsk, int? crHvcpu, string template, string forceMemo, string hv1gbPanelVersion, bool? panelInstall, bool? crSsd1, bool? crHvAllSystem)
        {
            return VdsCreateGet(token_, type, crHvmem, crHvdsk, crHvcpu, template, forceMemo, hv1gbPanelVersion, panelInstall, crSsd1, crHvAllSystem);
        }

        public virtual string VdsDeleteGet(NancyContext context, string token_, int? key_, bool? forceCritical)
        {
            return VdsDeleteGet(token_, key_, forceCritical);
        }

        public virtual string VdsIpAddGet(NancyContext context, string token_, int? key_)
        {
            return VdsIpAddGet(token_, key_);
        }

        public virtual string VdsIpDeleteGet(NancyContext context, string token_, int? key_, string delIp)
        {
            return VdsIpDeleteGet(token_, key_, delIp);
        }

        public virtual List<InlineResponse20011> VdsIpListGet(NancyContext context, string token_, int? key_)
        {
            return VdsIpListGet(token_, key_);
        }

        public virtual List<InlineResponse20012> VdsListGet(NancyContext context, string token_, int? key_)
        {
            return VdsListGet(token_, key_);
        }

        public virtual string VdsRebootGet(NancyContext context, string token_, int? key_)
        {
            return VdsRebootGet(token_, key_);
        }

        public virtual List<InlineResponse20013> VdsTemplatesListGet(NancyContext context, string token_)
        {
            return VdsTemplatesListGet(token_);
        }

        public virtual string VdsUpdateGet(NancyContext context, string token_, int? key_, int? crHvmem, int? crHvdsk, int? crHvcpu)
        {
            return VdsUpdateGet(token_, key_, crHvmem, crHvdsk, crHvcpu);
        }

        protected abstract string VdsCreateGet(string token_, string type, int? crHvmem, int? crHvdsk, int? crHvcpu, string template, string forceMemo, string hv1gbPanelVersion, bool? panelInstall, bool? crSsd1, bool? crHvAllSystem);

        protected abstract string VdsDeleteGet(string token_, int? key_, bool? forceCritical);

        protected abstract string VdsIpAddGet(string token_, int? key_);

        protected abstract string VdsIpDeleteGet(string token_, int? key_, string delIp);

        protected abstract List<InlineResponse20011> VdsIpListGet(string token_, int? key_);

        protected abstract List<InlineResponse20012> VdsListGet(string token_, int? key_);

        protected abstract string VdsRebootGet(string token_, int? key_);

        protected abstract List<InlineResponse20013> VdsTemplatesListGet(string token_);

        protected abstract string VdsUpdateGet(string token_, int? key_, int? crHvmem, int? crHvdsk, int? crHvcpu);
    }

}
